#ifndef ISSAC_H
#define ISSAC_H
//   _________
// _| isaac.h |_______________________________________________________________
// definitions for a random number generator
// Written by Bob Jenkins
// Re-formatted by John Cooke

#ifndef RNG_H
#include "rng.h"
#endif

//--[Array sizes]--
#define RANDSIZL   (8)  // I recommend 8 for crypto, 3 for simulations
#define RANDSIZ    (1<<RANDSIZL)

// the ISAAC random number generator
class ISAAC : public RNG
{
private:
	uint32 randcnt;
	uint32 randrsl[RANDSIZ];
	uint32 randmem[RANDSIZ];
	uint32 randa;
	uint32 randb;
	uint32 randc;

// this is part of initialisation
	void randinit();

// this is the generator
	void isaac();

	static RNGInfo *info;

public:

	void Seed (uint32) ;
	inline unsigned long lRand() {
		return (
			!randcnt--
			? (isaac(), randcnt=RANDSIZ-1, randrsl[randcnt])
			: randrsl[randcnt]
		);
	} ;

	const RNGInfo *Info() { return info; };

// we will have a constructor too
	ISAAC (uint32 seed) { Seed(seed); } ;
};

#endif


